@@ -185,6 +185,8 @@ def fastread(filename, x=0, y=1, batch = 0, read_last=0, df = None, plot = True)
     return df2
 
 
+#=====================================================================
+
 def coincidences(filename, ch0, ch1,  tree = False):
     """
     """
@@ -200,7 +202,7 @@ def coincidences(filename, ch0, ch1,  tree = False):
     #*************
     df = general.pd_read_table(filename, sort = True)
     #print(df.dtypes)
-    print(df)
+    #print(df)
     chan_available = df['ch'].unique()
     print("D... channels available:", chan_available )
     print("D... x        present:",df['pu'].unique() )
@@ -209,10 +211,65 @@ def coincidences(filename, ch0, ch1,  tree = False):
     print("D... Emin            :",df['E'].min() )
     print("D... Emax            :",df['E'].max() )
 
-    if (ch0 in chan_available) or (ch1 in chan_available):
+    if not((ch0 in chan_available) and (ch1 in chan_available)):
         print(f"X... channels {ch0} OR {ch1} not in the file !!")
         sys.exit(1)
 
+    #-------------------------here shold be something to check double events in a detector
+
+    #-------------------------here shold be something to check double events in a detector
+
+    #**************** select
+    df1 = general.select_channels(df, [ch0,ch1] , delete_original = True)
+
+    n_ch0 = len(df1.loc[ df1['ch']==ch0  ])
+    n_ch1 = len(df1.loc[ df1['ch']==ch1  ])
+    print(f"D... # events {n_ch0} and {n_ch1}")
+
+
+    #df1['time'] = np.where( df1['ch']==ch1,  df1['time'] ,  df1['time']+5.0)
+
+
+    TIMESHIFT = 2
+
+    df1.loc[ df1['ch'] ==ch1, 'time'] += TIMESHIFT*1e-6 # ADD  x us to ch1 time
+    df1 = df1.sort_values(by="time")
+    df1.reset_index(inplace=True, drop=True)
+
+
+    # df1.drop( df[df. < 50].index, inplace=True)
+    #*************  dt us    and next_E
+    df1 = general.enhance_by_dtus_and_next_E(df1)
+
+
+    df1.drop(df1.tail(1).index,inplace=True) # drop last n rows
+
+
+    #drop all longer than
+    df1 = df1.loc[ df1['dtus']<TIMESHIFT*2 ]
+
+    print(df1)
+
+    print(f"D... RANDOM EVENTS starting with ch=={ch1}: ",len(df1.loc[ df1['ch']==ch1] ) )
+    print(f"D... ZERO E EVENTS                       : ",len(df1.loc[ df1['E']==0] ) )
+    print(f"D... ZERO E EVENTS                       : ",len(df1.loc[ df1['next_E']==0] ) )
+    print()
+    print(f"D... N{ch0}={n_ch0} , N{ch1}={n_ch1},  COIN# {ch0}x{ch1} = {len(df1)} ... {100*len(df1)/min(n_ch0,n_ch1):.2f} %" )
+
+    print(f"D... dtus statistics:           MEAN = {df1['dtus'].mean():.2f}  STD={df1['dtus'].std():.2f} us" )
+    print()
+
+    bname = os.path.splitext(filename)[0]  #basename
+    #--------------- histogram coincidence time
+    hname = general.generate_hname(filename, ch0)+f"_coin{ch1}"
+    outfile = bname+f"_{ch0}_coin{ch1}"+".txt"
+    #*************
+    his = general.column_to_histo(  df1['dtus']  , binmax=int(2*TIMESHIFT*100), himax=2*TIMESHIFT, savename= outfile, hname = hname, writeondisk = True, writetxt = False) # 500 for 2us is exactly channel by channel
+
+
+    if tree:
+        df1.rename(columns = {'E':f'E{ch0}','next_E':f'E{ch1}'}, inplace = True)
+        general.save_to_tree( df1, filename, modname=f"coin_{ch0}_{ch1}" )
 
     return
 
@@ -277,9 +334,9 @@ def eva_cut_time(filename,  chan, od=0, do=9999999 ,   tree = False):
 
     if (od!=0) and (do<9999999):
         print()
-        print(f"D...  selecting events for  channel {chan} from {od}s to {do}s")
-        print(f"D...  selecting events for  channel {chan} from {od}s to {do}s")
-        print(f"D...  selecting events for  channel {chan} from {od}s to {do}s")
+        print(f"D...  selecting events    from {od}s to {do}s")
+        print(f"D...  selecting events    from {od}s to {do}s")
+        print(f"D...  selecting events    from {od}s to {do}s")
         print()
         df1 = df[ (df.time>od)&(df.time<do)  ].copy() # copy ELSE warning....
         df1.reset_index(inplace=True, drop=True)
@@ -293,6 +350,8 @@ def eva_cut_time(filename,  chan, od=0, do=9999999 ,   tree = False):
     else:
         df1 = df
 
+
+
     #---------------------------------------------------DF1 ---------------------------
     #************* reduce the DF
     df1 = general.select_channels(df1, [chan] , delete_original = True)
@@ -326,12 +385,18 @@ def eva_cut_time(filename,  chan, od=0, do=9999999 ,   tree = False):
 
 
     # not final
-    stat_columns = ['ch','rate','realT','liveT','deadT','satT','min_dtus','Nzero','Npu','Ntot','Nsat','DT%','DT+S%']
+    stat_columns = ['ch','rate','realT','liveT','deadT','satT','min_dtus','Nzero','Npu','Ndz','Ntot','Nsat','DT%','DT+S%']
     df_stat = pd.DataFrame( np.nan, index=[0], columns = stat_columns )
     df_stat['ch'] = chan
     df_stat['Nsat'] = satu_n_ch
     df_stat['satT'] = satu_DT_ch/1e+6
     df_stat['min_dtus'] = df1.loc[df1['dtus']!=0]['dtus'].min()
+    #print( df1.loc[df1['dtus']!=0]['dtus'].min())
+    #print( df1['dtus'].min())
+    if df_stat.iloc[-1]['min_dtus'] != df1['dtus'].min():
+        if len(df1.loc[ df1['dtus']==0] )!=1:
+            print("X... some error in creation of dtus - i need to debug it")
+            sys.exit(1)
     df_stat['Npu'] = pileups_ch
 
 
@@ -382,7 +447,7 @@ def eva_cut_time(filename,  chan, od=0, do=9999999 ,   tree = False):
 
 
     #*************   one channel; time span
-    len_dfzero,_ = general.pd_detect_zeroes(df1, chan) # df1[ (df1.E==0) ]
+    len_dfzero,len_dzeroes,len_szeroes,len_izeroes = general.pd_detect_zeroes(df1, chan) # df1[ (df1.E==0) ]
 
 
 
@@ -450,7 +515,10 @@ def eva_cut_time(filename,  chan, od=0, do=9999999 ,   tree = False):
     df_stat['Ntot'] = len(df1)
     df_stat['iDTavg'] = 100*( (1/rate) * satu_n_ch )/dift   #  average dt * nsatur
     df_stat['iDTmin'] = 100*( df_stat['min_dtus']*1e-6 * satu_n_ch )/dift  #  mintime * nsatur
-#    df_stat['tSAT'] =
+    #    df_stat['tSAT'] =
+
+
+    df_stat['Ndz'] = len_dzeroes+len_szeroes+len_izeroes
 
 #    df_stat = df_stat.round( decimals = 2 )
     df_stat = df_stat.round( decimals = 2 )
